@@ -13,10 +13,10 @@ pub trait Endpoint: Send + Sync + 'static {
     async fn call(&self, req: Request) -> Self::Output;
 }
 
-struct FnEndpoint<F>(F);
+struct SyncFnEndpoint<F>(F);
 
 #[async_trait::async_trait]
-impl<F, R> Endpoint for FnEndpoint<F>
+impl<F, R> Endpoint for SyncFnEndpoint<F>
 where
     F: Fn(Request) -> R + Send + Sync + 'static,
     R: IntoResponse,
@@ -28,14 +28,30 @@ where
     }
 }
 
+struct AsyncFnEndpoint<F>(F);
+
+#[async_trait::async_trait]
+impl<F, Fut, R> Endpoint for AsyncFnEndpoint<F>
+where
+    F: Fn(Request) -> Fut + Sync + Send + 'static,
+    Fut: Future<Output = R> + Send,
+    R: IntoResponse,
+{
+    type Output = R;
+
+    async fn call(&self, req: Request) -> Self::Output {
+        (self.0)(req).await
+    }
+}
+
 /// Create an endpoint with a function.
 ///
 /// # Example
 ///
 /// ```
-/// use poem::{fn_endpoint, http::Method, Endpoint, Request};
+/// use poem::{endpoint::make_sync, http::Method, Endpoint, Request};
 ///
-/// let ep = fn_endpoint(|req| req.method().to_string());
+/// let ep = make_sync(|req| req.method().to_string());
 ///
 /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
 /// let resp = ep
@@ -44,12 +60,37 @@ where
 /// assert_eq!(resp, "GET");
 /// # });
 /// ```
-pub fn fn_endpoint<F, R>(f: F) -> impl Endpoint<Output = R>
+pub fn make_sync<F, R>(f: F) -> impl Endpoint<Output = R>
 where
     F: Fn(Request) -> R + Send + Sync + 'static,
     R: IntoResponse,
 {
-    FnEndpoint(f)
+    SyncFnEndpoint(f)
+}
+
+/// Create an endpoint with a asyncness function.
+///
+/// # Example
+///
+/// ```
+/// use poem::{endpoint::make, http::Method, Endpoint, Request};
+///
+/// let ep = make(|req| async move { req.method().to_string() });
+///
+/// # tokio::runtime::Runtime::new().unwrap().block_on(async {
+/// let resp = ep
+///     .call(Request::builder().method(Method::GET).finish())
+///     .await;
+/// assert_eq!(resp, "GET");
+/// # });
+/// ```
+pub fn make<F, Fut, R>(f: F) -> impl Endpoint<Output = R>
+where
+    F: Fn(Request) -> Fut + Send + Sync + 'static,
+    Fut: Future<Output = R> + Send,
+    R: IntoResponse,
+{
+    AsyncFnEndpoint(f)
 }
 
 #[async_trait::async_trait]
@@ -70,8 +111,20 @@ impl<T: Endpoint + ?Sized> Endpoint for Arc<T> {
     }
 }
 
+/// An owned dynamically typed `Endpoint` for use in cases where you can’t
+/// statically type your result or need to add some indirection.
+pub type BoxEndpoint<T> = Box<dyn Endpoint<Output = T>>;
+
 /// Extension trait for [`Endpoint`].
 pub trait EndpointExt: Endpoint {
+    /// Wrap the endpoint in a Box.
+    fn boxed(self) -> BoxEndpoint<Self::Output>
+    where
+        Self: Sized + 'static,
+    {
+        Box::new(self)
+    }
+
     /// Use middleware to transform this endpoint.
     ///
     /// # Example
@@ -217,6 +270,7 @@ impl<T: Endpoint> EndpointExt for T {}
 #[cfg(test)]
 mod test {
     use crate::{
+        endpoint::make_sync,
         http::{Method, StatusCode},
         *,
     };
@@ -224,7 +278,7 @@ mod test {
     #[tokio::test]
     async fn test_before() {
         assert_eq!(
-            fn_endpoint(|req| req.method().to_string())
+            make_sync(|req| req.method().to_string())
                 .before(|mut req| async move {
                     req.set_method(Method::POST);
                     req
@@ -238,7 +292,7 @@ mod test {
     #[tokio::test]
     async fn test_after() {
         assert_eq!(
-            fn_endpoint(|_| "abc")
+            make_sync(|_| "abc")
                 .after(|_| async { "def" })
                 .call(Request::default())
                 .await,
@@ -249,7 +303,7 @@ mod test {
     #[tokio::test]
     async fn test_map_to_result() {
         assert_eq!(
-            fn_endpoint(|_| Response::builder().status(StatusCode::OK).body("abc"))
+            make_sync(|_| Response::builder().status(StatusCode::OK).body("abc"))
                 .map_to_result()
                 .call(Request::default())
                 .await
@@ -261,20 +315,18 @@ mod test {
             "abc"
         );
 
-        assert_eq!(
-            fn_endpoint(|_| Response::builder().status(StatusCode::BAD_REQUEST).finish())
-                .map_to_result()
-                .call(Request::default())
-                .await
-                .unwrap_err(),
-            Error::new(StatusCode::BAD_REQUEST)
-        );
+        let err = make_sync(|_| Response::builder().status(StatusCode::BAD_REQUEST).finish())
+            .map_to_result()
+            .call(Request::default())
+            .await
+            .unwrap_err();
+        assert_eq!(err.status(), StatusCode::BAD_REQUEST);
     }
 
     #[tokio::test]
     async fn test_map_to_response() {
         assert_eq!(
-            fn_endpoint(|_| Ok::<_, Error>("abc"))
+            make_sync(|_| Ok::<_, Error>("abc"))
                 .map_to_response()
                 .call(Request::default())
                 .await
@@ -286,7 +338,7 @@ mod test {
         );
 
         assert_eq!(
-            fn_endpoint(|_| Err::<(), Error>(Error::new(StatusCode::BAD_REQUEST)))
+            make_sync(|_| Err::<(), Error>(Error::new(StatusCode::BAD_REQUEST)))
                 .map_to_response()
                 .call(Request::default())
                 .await
@@ -298,7 +350,7 @@ mod test {
     #[tokio::test]
     async fn test_and_then() {
         assert_eq!(
-            fn_endpoint(|_| Ok("abc"))
+            make_sync(|_| Ok("abc"))
                 .and_then(|resp| async move { Ok(resp.to_string() + "def") })
                 .call(Request::default())
                 .await
@@ -306,20 +358,18 @@ mod test {
             "abcdef"
         );
 
-        assert_eq!(
-            fn_endpoint(|_| Err::<String, _>(Error::new(StatusCode::BAD_REQUEST)))
-                .and_then(|resp| async move { Ok(resp + "def") })
-                .call(Request::default())
-                .await
-                .unwrap_err(),
-            Error::new(StatusCode::BAD_REQUEST)
-        );
+        let err = make_sync(|_| Err::<String, _>(Error::new(StatusCode::BAD_REQUEST)))
+            .and_then(|resp| async move { Ok(resp + "def") })
+            .call(Request::default())
+            .await
+            .unwrap_err();
+        assert_eq!(err.status(), StatusCode::BAD_REQUEST);
     }
 
     #[tokio::test]
     async fn test_map_ok() {
         assert_eq!(
-            fn_endpoint(|_| Ok("abc"))
+            make_sync(|_| Ok("abc"))
                 .map_ok(|resp| async move { resp.to_string() + "def" })
                 .call(Request::default())
                 .await
@@ -327,20 +377,18 @@ mod test {
             "abcdef"
         );
 
-        assert_eq!(
-            fn_endpoint(|_| Err::<String, Error>(Error::new(StatusCode::BAD_REQUEST)))
-                .map_ok(|resp| async move { resp.to_string() + "def" })
-                .call(Request::default())
-                .await
-                .unwrap_err(),
-            Error::new(StatusCode::BAD_REQUEST)
-        );
+        let err = make_sync(|_| Err::<String, Error>(Error::new(StatusCode::BAD_REQUEST)))
+            .map_ok(|resp| async move { resp.to_string() + "def" })
+            .call(Request::default())
+            .await
+            .unwrap_err();
+        assert_eq!(err.status(), StatusCode::BAD_REQUEST);
     }
 
     #[tokio::test]
     async fn test_map_err() {
         assert_eq!(
-            fn_endpoint(|_| Ok("abc"))
+            make_sync(|_| Ok("abc"))
                 .map_err(|_| async move { Error::new(StatusCode::BAD_GATEWAY) })
                 .call(Request::default())
                 .await
@@ -348,13 +396,77 @@ mod test {
             "abc"
         );
 
+        let err = make_sync(|_| Err::<String, Error>(Error::new(StatusCode::BAD_REQUEST)))
+            .map_err(|_| async move { Error::new(StatusCode::BAD_GATEWAY) })
+            .call(Request::default())
+            .await
+            .unwrap_err();
+        assert_eq!(err.status(), StatusCode::BAD_GATEWAY);
+    }
+}
+
+/// Represents a type that can convert into endpoint.
+pub trait IntoEndpoint {
+    /// Represents the endpoint type.
+    type Endpoint: Endpoint;
+
+    /// Converts this object into endpoint.
+    fn into_endpoint(self) -> Self::Endpoint;
+}
+
+impl<T: Endpoint> IntoEndpoint for T {
+    type Endpoint = T;
+
+    fn into_endpoint(self) -> Self::Endpoint {
+        self
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::{
+        endpoint::make_sync,
+        http::Uri,
+        route::{route, Route},
+    };
+
+    #[tokio::test]
+    async fn test_into_endpoint() {
+        struct MyEndpointFactory;
+
+        impl IntoEndpoint for MyEndpointFactory {
+            type Endpoint = Route;
+
+            fn into_endpoint(self) -> Self::Endpoint {
+                let mut app = route();
+                app.at("/a").get(make_sync(|_| "a"));
+                app.at("/b").get(make_sync(|_| "b"));
+                app
+            }
+        }
+
+        let mut app = route();
+        app.nest("/api", MyEndpointFactory);
+
         assert_eq!(
-            fn_endpoint(|_| Err::<String, Error>(Error::new(StatusCode::BAD_REQUEST)))
-                .map_err(|_| async move { Error::new(StatusCode::BAD_GATEWAY) })
-                .call(Request::default())
+            app.call(Request::builder().uri(Uri::from_static("/api/a")).finish())
                 .await
-                .unwrap_err(),
-            Error::new(StatusCode::BAD_GATEWAY)
+                .take_body()
+                .into_string()
+                .await
+                .unwrap(),
+            "a"
+        );
+
+        assert_eq!(
+            app.call(Request::builder().uri(Uri::from_static("/api/b")).finish())
+                .await
+                .take_body()
+                .into_string()
+                .await
+                .unwrap(),
+            "b"
         );
     }
 }
